# Copyright (C) 2004-2025, Stefan Kanthak <stefan.kanthak@nexgo.de>

# * The software is provided "as is" without any warranty, neither express
#   nor implied.
# * In no event will the author be held liable for any damage(s) arising
#   from the use of the software.
# * Redistribution of the software is allowed only in unmodified form.
# * Permission is granted to use the software solely for personal private
#   and non-commercial purposes.
# * An individuals use of the software in his or her capacity or function
#   as an agent, (independent) contractor, employee, member or officer of
#   a business, corporation or organization (commercial or non-commercial)
#   does not qualify as personal private and non-commercial purpose.
# * Without written approval from the author the software must not be used
#   for a business, for commercial, corporate, governmental, military or
#   organizational purposes of any kind, or in a commercial, corporate,
#   governmental, military or organizational environment of any kind.

.SUFFIXES:

tempest: TEMPEST.DOS TEMPEST.EXE
	"$(MAKEDIR)\TEMPEST.EXE"

clobber: clean
	Erase TEMPEST.EXE

clean:
	Erase TEMPEST.DOS TEMPEST.OBJ

TEMPEST.EXE: TEMPEST.OBJ # TEMPEST.DOS
	LINK.EXE /LINK /BREPRO /DEFAULTLIB:advapi32 /DEFAULTLIB:kernel32 /DEFAULTLIB:user32 /ENTRY:wmainCRTStartup /FIXED /LARGEADDRESSAWARE /LAST:.data /MERGE:.rdata=.const /MERGE:.text=.code /NOCOFFGRPINFO /NOLOGO /NXCOMPAT /OPT:REF /OSVERSION:6.0 /OUT:$@ /RELEASE /STUB:TEMPEST.DOS /SUBSYSTEM:CONSOLE /SWAPRUN:CD,NET /VERSION:0.815 $**

TEMPEST.OBJ: TEMPEST.MAK
	CL.EXE /Brepro /c /Fo$@ /GA /GF /Gw /Gy /nologo /O1 /Oi /Tc<< /W4 /wd4090 /we4013 /Zl
// Copyright (C) 2004-2025, Stefan Kanthak <stefan.kanthak@nexgo.de>

// https://msdn.microsoft.com/en-us/library/7f0aews7.aspx

#pragma comment(compiler)

// https://msdn.microsoft.com/en-us/library/aa383732.aspx

#define STRICT
#define UNICODE
#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <sddl.h>
#include <aclapi.h>

// https://msdn.microsoft.com/en-us/library/hh916382.aspx

__declspec(safebuffers)
BOOL	CDECL	PrintConsole(HANDLE hConsole, [SA_FormatString(Style="printf")] LPCWSTR lpFormat, ...)
{
	WCHAR	szOutput[1025];
	DWORD	dwOutput;
	DWORD	dwConsole;

	// https://msdn.microsoft.com/en-us/library/kb57fad8.aspx

	va_list	vaInput;
	va_start(vaInput, lpFormat);

	// https://msdn.microsoft.com/en-us/library/ms647551.aspx

	dwOutput = wvsprintf(szOutput, lpFormat, vaInput);

	va_end(vaInput);

	if (dwOutput == 0UL)
		return FALSE;

	// https://msdn.microsoft.com/en-us/library/ms687401.aspx

	if (!WriteConsole(hConsole, szOutput, dwOutput, &dwConsole, NULL))
		return FALSE;

	return dwConsole == dwOutput;
}

// https://msdn.microsoft.com/en-us/library/aa374912.aspx

typedef	struct	_ace
{
	// https://msdn.microsoft.com/en-us/library/aa374919.aspx

	ACE_HEADER	Header;

	// https://msdn.microsoft.com/en-us/library/aa374892.aspx

	ACCESS_MASK	Mask;

	// https://msdn.microsoft.com/en-us/library/aa379594.aspx

	SID	Trustee;
} ACE;

static	struct	_acl
{
	// https://msdn.microsoft.com/en-us/library/aa374931.aspx

	ACL	acl;

	ACE	ace;
} dacl = {{ACL_REVISION, 0U, sizeof(dacl), 1U, 0U},
	// (A;CIOI;0x1301FF;;;OW)
          {{ACCESS_ALLOWED_ACE_TYPE, CONTAINER_INHERIT_ACE | OBJECT_INHERIT_ACE, sizeof(ACE)},
            SYNCHRONIZE | READ_CONTROL | DELETE | FILE_WRITE_ATTRIBUTES | FILE_READ_ATTRIBUTES | FILE_DELETE_CHILD | FILE_TRAVERSE | FILE_WRITE_EA | FILE_READ_EA | FILE_ADD_SUBDIRECTORY | FILE_ADD_FILE | FILE_LIST_DIRECTORY,
            {SID_REVISION, ANYSIZE_ARRAY, SECURITY_CREATOR_SID_AUTHORITY, SECURITY_CREATOR_OWNER_RIGHTS_RID}}},
  sacl = {{ACL_REVISION, 0U, sizeof(sacl), 1U, 0U},
	// (ML;CIOI;NRNWNX;;;ME)
          {{SYSTEM_MANDATORY_LABEL_ACE_TYPE, CONTAINER_INHERIT_ACE | OBJECT_INHERIT_ACE, sizeof(ACE)},
            SYSTEM_MANDATORY_LABEL_NO_EXECUTE_UP | SYSTEM_MANDATORY_LABEL_NO_READ_UP | SYSTEM_MANDATORY_LABEL_NO_WRITE_UP,
            {SID_REVISION, ANYSIZE_ARRAY, SECURITY_MANDATORY_LABEL_AUTHORITY, SECURITY_MANDATORY_MEDIUM_RID}}};

// https://msdn.microsoft.com/en-us/library/aa379561.aspx
// https://msdn.microsoft.com/en-us/library/aa379566.aspx

const	SECURITY_DESCRIPTOR	sd = {SECURITY_DESCRIPTOR_REVISION,
				      0,
#if 0
				      SE_DACL_PRESENT | SE_DACL_PROTECTED | SE_SACL_PRESENT | SE_SACL_PROTECTED,
#else
				      SE_DACL_PRESENT | SE_DACL_PROTECTED | SE_SACL_PRESENT,
#endif
				      (SID *) NULL,
				      (SID *) NULL,
				      &sacl.acl,
				      &dacl.acl};

// https://msdn.microsoft.com/en-us/library/aa379560.aspx

const	SECURITY_ATTRIBUTES	sa = {sizeof(sa),
				      &sd,
				      FALSE};

// https://msdn.microsoft.com/en-us/library/f9t8842e.aspx

__declspec(noreturn)
VOID	CDECL	wmainCRTStartup(VOID)
{
	// https://msdn.microsoft.com/en-us/library/bb394727.aspx

	struct	_token_mandatory_label
	{
		// https://msdn.microsoft.com/en-us/library/aa379595.aspx

		SID_AND_ATTRIBUTES	Label;

		// https://msdn.microsoft.com/en-us/library/aa379594.aspx

		SID	Sid;
	} tml;

	// https://msdn.microsoft.com/en-us/library/aa379561.aspx

	SECURITY_DESCRIPTOR	*lpSD;

	LPWSTR	lpSDDL;
	DWORD	dwError = ERROR_SUCCESS;
	DWORD	dwBuffer;
	WCHAR	szBuffer[MAX_PATH + 2];
	WCHAR	szUnique[MAX_PATH];

	// https://msdn.microsoft.com/en-us/library/ms683180.aspx
	// https://msdn.microsoft.com/en-us/library/ms683183.aspx

	UINT	uiUnique = ((GetCurrentProcessId() + GetCurrentThreadId()) >> 1) & 65535U;

	// https://msdn.microsoft.com/en-us/library/ms683179.aspx

	HANDLE	hProcess = GetCurrentProcess();
	HANDLE	hToken;

	// https://msdn.microsoft.com/en-us/library/ms683231.aspx

	HANDLE	hConsole = GetStdHandle(STD_ERROR_HANDLE);

	if (hConsole == INVALID_HANDLE_VALUE)
		dwError = GetLastError();
	else
	{	// https://msdn.microsoft.com/en-us/library/aa379295.aspx

		if (!OpenProcessToken(hProcess, TOKEN_QUERY, &hToken))
			PrintConsole(hConsole,
			             L"%ls() returned error %lu\n",
			             L"OpenProcessToken", dwError = GetLastError());
		else
		{	// https://msdn.microsoft.com/en-us/library/aa446671.aspx
			// https://msdn.microsoft.com/en-us/library/aa379626.aspx

			if (!GetTokenInformation(hToken,
			                         TokenIntegrityLevel,
			                         &tml,
			                         sizeof(tml),
			                         &dwBuffer))
				PrintConsole(hConsole,
				             L"%ls() returned error %lu for information class %lu\n",
				             L"GetTokenInformation", dwError = GetLastError(), TokenIntegrityLevel);
			else
			{	// https://msdn.microsoft.com/en-us/library/aa376404.aspx

				if (!CopySid(sizeof(sacl.ace.Trustee), &sacl.ace.Trustee, tml.Label.Sid))
					PrintConsole(hConsole,
					             L"%ls() returned error %lu\n",
					             L"CopySid", dwError = GetLastError());
				else
				{	// https://msdn.microsoft.com/en-us/library/aa364992.aspx

					dwBuffer = GetTempPath(sizeof(szBuffer) / sizeof(*szBuffer), szBuffer);

					if (dwBuffer == 0UL)
						PrintConsole(hConsole,
						             L"%ls() returned error %lu\n",
						             L"GetTempPath", dwError = GetLastError());
					else
					{
						PrintConsole(hConsole,
						             L"%ls() returned pathname \'%ls\' of %lu characters\n",
						             L"GetTempPath", szBuffer, dwBuffer);

						do
						{	// https://msdn.microsoft.com/en-us/library/aa364991.aspx

							if (GetTempFileName(szBuffer, L"tmp", uiUnique, szUnique) == 0U)
								PrintConsole(hConsole,
								             L"%ls() returned error %lu\n",
								             L"GetTempFileName", dwError = GetLastError());
							else
								// https://msdn.microsoft.com/en-us/library/aa363855.aspx

								if (!CreateDirectory(szUnique, &sa))
								{
									PrintConsole(hConsole,
									             L"%ls() returned error %lu for pathname \'%ls\'\n",
									             L"CreateDirectory", dwError = GetLastError(), szUnique);

									if (dwError == ERROR_ALREADY_EXISTS)
										continue;
								}
								else
								{	// https://msdn.microsoft.com/en-us/library/aa446645.aspx

									dwError = GetNamedSecurityInfo(szUnique,
									                               SE_FILE_OBJECT,
									                               OWNER_SECURITY_INFORMATION | GROUP_SECURITY_INFORMATION | DACL_SECURITY_INFORMATION | LABEL_SECURITY_INFORMATION,
									                               (SID **) NULL,
									                               (SID **) NULL,
									                               (ACL **) NULL,
									                               (ACL **) NULL,
									                               &lpSD);

									if (dwError != ERROR_SUCCESS)
										PrintConsole(hConsole,
										             L"%ls() returned error %lu\n",
										             L"GetNamedSecurityInfo", dwError);
									else
										// https://msdn.microsoft.com/en-us/library/aa376397.aspx

										if (!ConvertSecurityDescriptorToStringSecurityDescriptor(lpSD,
										                                                         SDDL_REVISION_1,
										                                                         OWNER_SECURITY_INFORMATION | GROUP_SECURITY_INFORMATION | DACL_SECURITY_INFORMATION | SACL_SECURITY_INFORMATION | LABEL_SECURITY_INFORMATION,
										                                                         &lpSDDL,
										                                                         (LPDWORD) NULL))
											PrintConsole(hConsole,
											             L"%ls() returned error %lu\n",
											             L"ConvertSecurityDescriptorToStringSecurityDescriptor", dwError = GetLastError());
										else
										{
											PrintConsole(hConsole,
											             L"Subdirectory \'%ls\' created with security descriptor \'%ls\'\n",
											             szUnique, lpSDDL);

											// https://msdn.microsoft.com/en-us/library/aa366730.aspx

											if (LocalFree(lpSDDL) != NULL)
												PrintConsole(hConsole,
												             L"%ls() returned error %lu\n",
												             L"LocalFree", dwError = GetLastError());
										}

									// https://msdn.microsoft.com/en-us/library/aa364991.aspx

									if (GetTempFileName(szUnique, L"tmp", 0U, szBuffer) == 0U)
										PrintConsole(hConsole,
										             L"%ls() returned error %lu\n",
										             L"GetTempFileName", dwError = GetLastError());
									else
									{	// https://msdn.microsoft.com/en-us/library/aa446645.aspx

										dwError = GetNamedSecurityInfo(szBuffer,
										                               SE_FILE_OBJECT,
										                               OWNER_SECURITY_INFORMATION | GROUP_SECURITY_INFORMATION | DACL_SECURITY_INFORMATION | LABEL_SECURITY_INFORMATION,
										                               (SID **) NULL,
										                               (SID **) NULL,
										                               (ACL **) NULL,
										                               (ACL **) NULL,
										                               &lpSD);

										if (dwError != ERROR_SUCCESS)
											PrintConsole(hConsole,
											             L"%ls() returned error %lu\n",
											             L"GetNamedSecurityInfo", dwError);
										else
											// https://msdn.microsoft.com/en-us/library/aa376397.aspx

											if (!ConvertSecurityDescriptorToStringSecurityDescriptor(lpSD,
											                                                         SDDL_REVISION_1,
											                                                         OWNER_SECURITY_INFORMATION | GROUP_SECURITY_INFORMATION | DACL_SECURITY_INFORMATION | SACL_SECURITY_INFORMATION | LABEL_SECURITY_INFORMATION,
											                                                         &lpSDDL,
											                                                         (LPDWORD) NULL))
												PrintConsole(hConsole,
												             L"%ls() returned error %lu\n",
												             L"ConvertSecurityDescriptorToStringSecurityDescriptor", dwError = GetLastError());
											else
											{
												PrintConsole(hConsole,
												             L"File \'%ls\' created with security descriptor \'%ls\'\n",
												             szBuffer, lpSDDL);

												// https://msdn.microsoft.com/en-us/library/aa366730.aspx

												if (LocalFree(lpSDDL) != NULL)
													PrintConsole(hConsole,
													             L"%ls() returned error %lu\n",
													             L"LocalFree", dwError = GetLastError());
											}

										// https://msdn.microsoft.com/en-us/library/aa363915.aspx

										if (!DeleteFile(szBuffer))
											PrintConsole(hConsole,
											             L"%ls() returned error %lu\n",
											             L"DeleteFile", dwError = GetLastError());
									}

									// https://msdn.microsoft.com/en-us/library/aa365488.aspx

									if (!RemoveDirectory(szUnique))
										PrintConsole(hConsole,
										             L"%ls() returned error %lu\n",
										             L"RemoveDirectory", dwError = GetLastError());
								}

							break;
						}
						while (--uiUnique > 0U);
					}
				}
			}

			// https://msdn.microsoft.com/en-us/library/ms724211.aspx

			if (!CloseHandle(hToken))
				PrintConsole(hConsole,
				             L"%ls() returned error %lu\n",
				             L"CloseHandle", GetLastError());
		}

		// https://msdn.microsoft.com/en-us/library/ms724211.aspx

		if (!CloseHandle(hConsole))
			PrintConsole(hConsole,
			             L"%ls() returned error %lu\n",
			             L"CloseHandle", GetLastError());
	}

	// https://msdn.microsoft.com/en-us/library/ms682658.aspx

	ExitProcess(dwError);
}
<<NOKEEP

TEMPEST.OMF: TEMPEST.MAK
	ML.EXE /Brepro /Fo$@ /nologo /omf /Ta<< /W3 /X
; Copyright (C) 2004-2025, Stefan Kanthak <stefan.kanthak@nexgo.de>

	.model	tiny

; https://support.microsoft.com/en-us/kb/65122

e_header struct				; DOS .EXE header
e_magic	word	5A4Dh			; magic number "MZ"
e_cblp	word	stop - header		; bytes on last page of file
e_cp	word	1			; pages in file
e_crlc	word	0			; number of relocation table entries
e_cparhdr word	sizeof header / 16	; size of header in paragraphs
e_minalloc word	0			; minimum extra paragraphs needed
e_maxalloc word	65535			; maximum extra paragraphs needed
e_ss	word	0			; initial (PSP relative) SS value
e_sp	word	stack - header		; initial (SS relative) SP value
e_csum	word	0			; checksum
e_ip	word	sizeof message		; initial (CS relative) IP value
e_cs	word	0			; initial (PSP relative) CS value
e_lfarlc word	sizeof header		; file address of relocation table
e_ovno	word	0			; overlay number
e_res	word	4 dup (0)		; reserved words
e_oemid	word	5719h			; OEM identifier
e_oeminfo word	2704h			; OEM information
e_res2	word	10 dup (0)		; reserved words
e_lfanew dword	0			; file address of new exe header
e_header ends

	.code

header	e_header <>

message	byte	"(C)opyright 2004-2025, Stefan Kanthak <stefan.kanthak@nexgo.de>", 0Dh, 0Ah, 07h, 24h

start:
	push	cs
	pop	ds
	xor	dx, dx
	mov	ah, 09h
	int	21h

	mov	ax, 4C01h
	int	21h
stop:
	dword	20 dup (?)
stack:
	end	start
<<NOKEEP

TEMPEST.DOS: TEMPEST.MAK
	CERTUTIL.EXE /DecodeHex /F /V << $@
4D 5A 90 00 01 00 00 00 04 00 00 00 FF FF 00 00   MZ..............
E0 00 00 00 43 00 00 00 40 00 00 00 00 00 00 00   ....C...@.......
00 00 00 00 19 57 04 27 00 00 00 00 00 00 00 00   .....W.'........
00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00   ................
28 43 29 6F 70 79 72 69 67 68 74 20 32 30 30 34   (C)opyright 2004
2D 32 30 32 35 2C 20 53 74 65 66 61 6E 20 4B 61   -2025, Stefan Ka
6E 74 68 61 6B 20 3C 73 74 65 66 61 6E 2E 6B 61   nthak <stefan.ka
6E 74 68 61 6B 40 6E 65 78 67 6F 2E 64 65 3E 0D   nthak@nexgo.de>.
0A 07 24 0E 1F 33 D2 B4 09 CD 21 B8 01 4C CD 21   .....3....!..L.!
<<NOKEEP
